#include <stdio.h>
#include <math.h>

double funct_A(const double x)
{
   return 2*x*x*x-4*x+1;
}

/* This is a general bisection method for functions whose 
   signature is f(const double x) */
double get_delta_root(double (*f)(const double),
		      double lb, 
		      double rb,
		      const double delta)
{
   double m=(lb+rb)/2;

   double f_lb=f(lb),
	  f_m=f(m);

   while (fabs(f_m)>=delta && 
          m!=lb     /* This further condition is mean to deal with 
                       the finite arithmetic of the computer:
                       if f(x) is too steep in a neighborhood 
                       of its root, when it may be the case that 
                       the computation reduces the interval 
                       to one single point (i.e., lb=rb), but 
                       |f(lb)| is still greater than delta. 
                       In these cases, lb is the best 
                       approximation for the root among the 
                       doubles */
         ) {
       if (f_lb*f_m<0) {
	 rb=m;
       } else {
	 lb=m;
         f_lb=f_m;	 
       }

       m=(lb+rb)/2;
       f_m=f(m);
   }

   return m;
}

int main(int argc, char *argv[])
{
   double delta=1.0;

   for (unsigned int i=0; i<6; i++) {
     double root=get_delta_root(funct_A, 0.0, 1.0, delta);

     printf("%f is a (%.0e)-root for f(x)=2*x^3-4*x+1: f(%f)=%.3e\n", 
  		   root, delta, root, funct_A(root));
     delta = delta/10;
   }

   return 0;
}
